#include "def_vst.h"

#define MAX_N  200       /* max. length of UFVs */


/*----------------------------------------------------------------------*/
int tokens_of(char *in, char *out[])
{   NAME("tokens_of()");
    int tc=0,i=0;
    char *cp;
    bool within_token = FALSE;

    cp = in;
    do
    {    if(within_token && (*cp==' ' || *cp=='\t'))
         {   within_token = FALSE;
             *cp=0;
         }
         else if(!within_token && *cp!=' ' && *cp!='\t')
         {    within_token = TRUE;
              out[tc++]=cp;
         }
    }    while(i++, *++cp);
    return tc;
}


int main(int argc, char *argv[])
{
    int      i,n,
             nf;                     /* # floats                    */
    char     *line_str,
             *cp;
    float    ufv[MAX_N];             /* unlabelled feature vector   */

    long     vec_cnt=0;              /* Anzahl der ufv's            */

    
    #define  MAXLL 2048
    char     *xfn,
             line[MAXLL];
    char     *token[MAXLL/2];

    FILE     *xfd;

    struct index_range
    {   int first;
        int last;
        struct index_range *next;
    };


    struct index_range  *first_range=0, *range, *prev_range=0;
    
    static char *synopsis =
"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n"
"                                       V1.0, 18/12/2006,  V.Strom  \n"
"                                                                   \n"
" usage: ufv_discard  n xfn    <ufv  >ufv                           \n"
"                                                                   \n"
" <ufv  unlabled feature vectors              (float[])             \n"
" n     vector length (nuber of floats)       (int)                 \n"
" xfn   file containing a list of vector indices which shall be     \n"
"       discarded i.e. not throughput.  Indices should be           \n"
"       separated by white space, ranges are allowed, too,          \n"
"       e.g. '0-13 15 91-99'  Note that counting starts with 0,     \n"
"       ranges are inclusive and no white space within a range.     \n"
"                                                                   \n"
"                                                                   \n"
"~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~\n";
    ARGS(2,"","");

    IARG(1,nf);
    if(nf<1 || nf >MAX_N)
        ERROR1("n is out of range [1:%d]",MAX_N);

    SARG(2,xfn);

    /*======================================================*/
    if(xfn)
    {   FOPEN(xfd, xfn, "r");
        while(fgets(line, MAXLL, xfd))
        {   cp = strrchr(line,'\n'); // delete newline if possible
            if(!cp)
                ERROR2("max line length %d exceeded in %s", MAXLL,xfn);
            *cp=0;
                
            n = tokens_of(line, token);
            for(i=0; i<n; i++)
            {   ALLOCATE(range, struct index_range, 1);
                if(sscanf(token[i], "%d-%d", &range->first, &range->last)==2)
                {   if(range->first<0 || range->first>range->last)
                        ERROR3("illegal range %d-%d in %s", range->first,
                                                            range->last, xfn);
                }
                else if(sscanf(token[i], "%d", &range->first)==1)
                {   range->last = range->first;
                    }
                else
                   ERROR2("format error in file %s: %s",xfn,token[i]);
                if(prev_range)
                {   if(prev_range->last > range->first)
                        ERROR3("indices in %s must be monotonic "
                               "%d >= %d", xfn, prev_range->last, range->first);
                }
                range->next=0;

                if(!first_range)
                {   first_range = range;
                    prev_range = range;
                }
                else
                {   prev_range->next = range;
                    prev_range = range;
                }
            }
        }
    }
    IF_DEBUG
    {   range=first_range;
        while(range)
        {   fprintf(stderr,"RANGE %d - %d\n", range->first, range->last);
            range=range->next;
        }
    }
    range=first_range;
    /*============ Hauptschleife: ==========================*/
    while(FREAD(ufv, float, EXACTLY, nf, stdin))
    {   if(range)
        {   if(vec_cnt>=range->first && vec_cnt<=range->last)
            {    vec_cnt++;
                 continue;
            }
            if(vec_cnt>range->last)
                range=range->next;
        }
        vec_cnt++;
        FWRITE(ufv, float, nf, stdout);
    }
    /*======================================================*/
    if(vec_cnt == 0)
        ERROR0("input file empty");

    return 0;
}

